//
//  FirstViewController.swift
//  VP3300_Test_App
//
//  Created by IDTech on 7/14/16.
//  Copyright © 2016 IDTech. All rights reserved.
//

import UIKit

var _lcdDisplayMode: Int32 = 0

extension Data {
    struct HexEncodingOptions: OptionSet {
        let rawValue: Int
        static let upperCase = HexEncodingOptions(rawValue: 1 << 0)
    }
    
    func hexEncodedString(options: HexEncodingOptions = []) -> String {
        let format = options.contains(.upperCase) ? "%02hhX" : "%02hhx"
        return map { String(format: format, $0) }.joined()
    }
}

extension String {
    func displayEntries() -> String{
        return self.replacingOccurrences(of: "AnyHashable(", with: "\n").replacingOccurrences(of: "):", with: ":")
    }
}


class FirstViewController: UIViewController, UITextFieldDelegate, IDT_VP3300_Delegate, UIAlertViewDelegate {
    
    
    
    var connection = Connection()
    
    @IBOutlet weak var connectionStatus: UILabel!
    @IBOutlet weak var resultsTextView: UITextView!
    @IBOutlet weak var dataTextView: UITextView!
    @IBOutlet var friendlyName: UITextField!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        
        self.friendlyName.delegate = self;
        
        
        IDT_VP3300.sharedController().delegate = self
        
        let tbvc = tabBarController as! TabBarController
        connection = tbvc.connection
    }
    
    // End the editing.
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        self.view.endEditing(true)
        return false
    }
    
    override func viewWillAppear(_ animated: Bool) {
        connectionStatus.text = connection.getTheStatus()
        connectionStatus.backgroundColor = connection.getTheBackgroundColor()
        resultsTextView.text = connection.getTheResultsText()
        dataTextView.text = connection.getTheDataText()
        friendlyName.text = IDT_VP3300.sharedController().device_getBLEFriendlyName()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func setConnectionStatus(_ status: String, backgroundColor: UIColor) {
        connection.setTheStatus(status)
        connectionStatus.text = connection.getTheStatus()
        
        connection.setTheBackgroundColor(backgroundColor)
        connectionStatus.backgroundColor = connection.getTheBackgroundColor()
        
        NotificationCenter.default.post(name: Notification.Name(rawValue: "load"), object: nil)
    }
    
    func deviceConnected() {
		//This will fire when device connects
        setConnectionStatus("Connected", backgroundColor: UIColor.green)
        
        connection.appendMessageToResultsField("VP3300 Connected\nFramework Version: \(IDT_Device.sdk_version())", field: resultsTextView)
    }
    
    func deviceDisconnected() {
		//This will fire when device disconnects
        setConnectionStatus("Disconnected", backgroundColor: UIColor.red)
    }
	
	func gen2Data(_ tlv: Data!){
		//This will receive TLV data
	}
    
    func plugStatusChange(_ deviceInserted: Bool) {
        if deviceInserted {
            connection.appendMessageToResultsField("Device attached. Attempting to connect...", field: resultsTextView)
            IDT_VP3300.sharedController().device_connectToAudioReader()
        } else {
            connection.appendMessageToResultsField("Device removed.", field: resultsTextView)
        }
    }
    
    func data(inOutMonitor data: Data!, incoming isIncoming: Bool) {
        if isIncoming {
            connection.appendMessageToDataField("IN: \(data.hexEncodedString())\n", field: dataTextView)
        } else {
            connection.appendMessageToDataField("\n====================\nOUT: \(data.hexEncodedString())\n", field: dataTextView)
        }
    }
    
    func swipeMSRData(_ cardData: IDTMSRData!) {
        NSLog("--MSR event received, type: \(cardData.event), data: \(cardData.encTrack1)")
        //NSLog("YO! card data", cardData.encryptedTags);
        switch cardData.event {
        case EVENT_MSR_CARD_DATA:
            switch cardData.captureEncodeType {
            case CAPTURE_ENCODE_TYPE_ISOABA:
                connection.appendMessageToResultsField("Encode Type: ISO/ABA", field: resultsTextView)
            case CAPTURE_ENCODE_TYPE_AAMVA:
                connection.appendMessageToResultsField("Encode Type: AA/MVA", field: resultsTextView)
            case CAPTURE_ENCODE_TYPE_Other:
                connection.appendMessageToResultsField("Encode Type: Other", field: resultsTextView)
            case CAPTURE_ENCODE_TYPE_Raw:
                connection.appendMessageToResultsField("Encode Type: Raw", field: resultsTextView)
            case CAPTURE_ENCODE_TYPE_JIS_I:
                connection.appendMessageToResultsField("Encode Type: CAPTURE_ENCODE_TYPE_JIS_I", field: resultsTextView)
            case CAPTURE_ENCODE_TYPE_JIS_II:
                connection.appendMessageToResultsField("Encode Type: CAPTURE_ENCODE_TYPE_JIS_II", field: resultsTextView)
            default:
                connection.appendMessageToResultsField("Encode Type: UNKWOWN", field: resultsTextView)
            }
            
            switch cardData.captureEncryptType {
            case CAPTURE_ENCRYPT_TYPE_AES:
                connection.appendMessageToResultsField("Encrypt Type: AES", field: resultsTextView)
            case CAPTURE_ENCRYPT_TYPE_TDES:
                connection.appendMessageToResultsField("Encrypt Type: TDES", field: resultsTextView)
            case CAPTURE_ENCRYPT_TYPE_NO_ENCRYPTION:
                connection.appendMessageToResultsField("Encrypt Type: NONE", field: resultsTextView)
            default:
                connection.appendMessageToResultsField("Encrypt Type: UNKNOWN", field: resultsTextView)
            }
            
            connection.appendMessageToResultsField("Full card data: \(cardData.cardData == nil ? "N/A" : cardData.cardData.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("Track 1: \(cardData.track1 == nil ? "N/A" : cardData.track1)", field: resultsTextView)
            connection.appendMessageToResultsField("Track 2: \(cardData.track2 == nil ? "N/A" : cardData.track2)", field: resultsTextView)
            connection.appendMessageToResultsField("Track 3: \(cardData.track3 == nil ? "N/A" : cardData.track3)", field: resultsTextView)
            connection.appendMessageToResultsField("Length Track 1: \(cardData.track1Length)", field: resultsTextView)
            connection.appendMessageToResultsField("Length Track 2: \(cardData.track2Length)", field: resultsTextView)
            connection.appendMessageToResultsField("Length Track 3: \(cardData.track3Length)", field: resultsTextView)
            connection.appendMessageToResultsField("Encoded Track 1: \(cardData.encTrack1 == nil ? "N/A" : cardData.encTrack1.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("Encoded Track 2: \(cardData.encTrack2 == nil ? "N/A" : cardData.encTrack2.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("Encoded Track 3: \(cardData.encTrack3 == nil ? "N/A" : cardData.encTrack3.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("Hash Track 1: \(cardData.hashTrack1 == nil ? "N/A" : cardData.hashTrack1.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("Hash Track 2: \(cardData.hashTrack2 == nil ? "N/A" : cardData.hashTrack2.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("Hash Track 3: \(cardData.hashTrack3 == nil ? "N/A" : cardData.hashTrack3.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("KSN: \(cardData.ksn == nil ? "N/A" : cardData.ksn.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("\nSessionID: \(cardData.sessionID == nil ? "N/A" : cardData.sessionID.hexEncodedString())", field: resultsTextView)
            connection.appendMessageToResultsField("\nReader Serial Number: \(cardData.rsn == nil ? "N/A" : cardData.rsn)", field: resultsTextView)
            connection.appendMessageToResultsField("\nRead Status: \(cardData.readStatus)", field: resultsTextView)
            
            if cardData.unencryptedTags != nil {
                connection.appendMessageToResultsField("Unencrypted Tags: \( cardData.unencryptedTags.description.displayEntries())", field: resultsTextView)
            }
            
            if cardData.encryptedTags != nil {
                connection.appendMessageToResultsField("Encrypted Tags: \(cardData.encryptedTags.description.displayEntries())", field: resultsTextView)
            }
            
            if cardData.maskedTags != nil {
                connection.appendMessageToResultsField("Masked Tags: \(cardData.maskedTags.description.displayEntries())", field: resultsTextView)
            }
            
            NSLog("Track 1: \(cardData.track1 == nil ? "N/A" : cardData.track1)")
            NSLog("Track 2: \(cardData.track2 == nil ? "N/A" : cardData.track2)")
            NSLog("Track 3: \(cardData.track3 == nil ? "N/A" : cardData.track3)")
            NSLog("Encoded Track 1: \(cardData.encTrack1 == nil ? "N/A" : cardData.encTrack1.hexEncodedString())")
            NSLog("Encoded Track 2: \(cardData.encTrack2 == nil ? "N/A" : cardData.encTrack2.hexEncodedString())")
            NSLog("Encoded Track 3: \(cardData.encTrack3 == nil ? "N/A" : cardData.encTrack3.hexEncodedString())")
            NSLog("Hash Track 1: \(cardData.hashTrack1 == nil ? "N/A" : cardData.hashTrack1.hexEncodedString())")
            NSLog("Hash Track 2: \(cardData.hashTrack2 == nil ? "N/A" : cardData.hashTrack2.hexEncodedString())")
            NSLog("Hash Track 3: \(cardData.hashTrack3 == nil ? "N/A" : cardData.hashTrack3.hexEncodedString())")
            NSLog("SessionID: \(cardData.sessionID == nil ? "N/A" : cardData.sessionID.hexEncodedString())")
            NSLog("nReader Serial Number: \(cardData.rsn == nil ? "N/A" : cardData.rsn)")
            NSLog("Read Status: \(cardData.readStatus)")
            NSLog("KSN: \(cardData.ksn == nil ? "N/A" : cardData.ksn.hexEncodedString())")
            
        case EVENT_MSR_CANCEL_KEY:
            connection.appendMessageToResultsField("(Event) MSR Cancel Key received: \(cardData.encTrack1)", field: resultsTextView)
            
        case EVENT_MSR_BACKSPACE_KEY:
            connection.appendMessageToResultsField("(Event) MSR Backspack Key received: \(cardData.encTrack1)", field: resultsTextView)
            
        case EVENT_MSR_ENTER_KEY:
            connection.appendMessageToResultsField("(Event) MSR Enter Key received: \(cardData.encTrack1)", field: resultsTextView)
            
        case EVENT_MSR_UNKNOWN:
            connection.appendMessageToResultsField("(Event) MSR unknown event, data: \(cardData.encTrack1)", field: resultsTextView)
        case EVENT_MSR_TIMEOUT:
            connection.appendMessageToResultsField("(Event) MSR Timeout",field: resultsTextView)
            
        default:
            break
            
        }
    }
    
	@objc func completeEMV() {
        NotificationCenter.default.post(name: Notification.Name(rawValue: "completeEMV"), object: nil)
    }
    
	@objc func startEMV() {
        NotificationCenter.default.post(name: Notification.Name(rawValue: "startEMV"), object: nil)
    }
    
    func emvTransactionData(_ emvData: IDTEMVData!, errorCode error: Int32) {
        var loopData = connection.getTheStressTestEMVLoopStatus()
        
        connection.appendMessageToResultsField("EMV transaction data response: \(IDT_VP3300.sharedController().device_getResponseCodeString(error))\n", field: resultsTextView)
        
        if emvData == nil {
            connection.appendMessageToResultsField("EMV TRANSACTION ERROR. Refer to EMV_RESULT_CODE_V2_response = \(error)", field: resultsTextView)
           
            
            
            return;
 
        }
        
        //
        
        if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_TIME_OUT {
            
        }
        
        if emvData.resultCodeV2 != EMV_RESULT_CODE_V2_NO_RESPONSE {
            connection.appendMessageToResultsField("EMV_RESULT_CODE_V2_RESPONSE: \(emvData.resultCodeV2.rawValue)", field: resultsTextView)
        }
        
        if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_GO_ONLINE {
            connection.appendMessageToResultsField("ONLINE REQUEST", field: resultsTextView)
            loopData = false
            
            if connection.getTheAutoCompleteStatus() {
                Timer.scheduledTimer(timeInterval: 0.5, target: self, selector: #selector(completeEMV), userInfo: nil, repeats: false)
            }
        }
        
        if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_START_TRANS_SUCCESS {
            loopData = false
            connection.appendMessageToResultsField("Start success: authentication required", field: resultsTextView)
        }
        
        if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_APPROVED || emvData.resultCodeV2 == EMV_RESULT_CODE_V2_APPROVED_OFFLINE {
            connection.appendMessageToResultsField("APPROVED", field: resultsTextView);
        }
        
        if emvData.resultCodeV2 == EMV_RESULT_CODE_V2_SWIPE_NON_ICC {
            connection.appendMessageToResultsField("MSR Data Captured", field: resultsTextView)
        }
        
        if emvData.cardType == 0 {
            connection.appendMessageToResultsField("CONTACT", field: resultsTextView)
        }
        
        if emvData.cardType == 1 {
            connection.appendMessageToResultsField("CONTACTLESS", field: resultsTextView)
        }
        
        if emvData.unencryptedTags != nil {
            connection.appendMessageToResultsField("Unencrypted Tags: \(emvData.unencryptedTags.description.displayEntries())", field: resultsTextView)
        }
        
        if emvData.encryptedTags != nil {
            connection.appendMessageToResultsField("Encrypted Tags: \(emvData.encryptedTags.description.displayEntries())", field: resultsTextView)
        }
        
        if emvData.maskedTags != nil {
            connection.appendMessageToResultsField("Masked Tags: \(emvData.maskedTags.description.displayEntries())", field: resultsTextView)
        }
        
        if emvData.hasAdvise {
            connection.appendMessageToResultsField("Response has advise request", field: resultsTextView)
        }
        
        if emvData.hasReversal {
            connection.appendMessageToResultsField("Response has reversal request", field: resultsTextView)
        }
        
        if loopData {
            Timer.scheduledTimer(timeInterval: 0.5, target: self, selector: #selector(startEMV), userInfo: nil, repeats: false)
        }
    }
    
	func _setResultViewText(str:String){
		connection.setTheResultsText(str)
		resultsTextView.text = str;
	}
	
	func setResultViewText(str:String){
		DispatchQueue.main.async { [unowned self] in
			self._setResultViewText(str: str)
		}
	}
	
    func lcdDisplay(_ mode: Int32, lines: [Any]!) {
        var str = ""
        _lcdDisplayMode = mode
        
        if lines != nil {
            for s in lines {
                str += s as! String
                str += "\n"
            }
        }
        
        switch mode {
        case 0x10:
			setResultViewText(str: str)
            NotificationCenter.default.post(name: Notification.Name(rawValue: "load"), object: nil)
        case 0x03:
			setResultViewText(str: str)
            NotificationCenter.default.post(name: Notification.Name(rawValue: "load"), object: nil)
        case 0x01, 0x02, 0x08:
            let alert = UIAlertView(title: "Please select", message: str, delegate: self, cancelButtonTitle: "Cancel", otherButtonTitles: "Ok", "")
            alert.alertViewStyle = .plainTextInput
            alert.show()
        default:
            break
        }
    }
    var _mode: EMV_PIN_MODE_Types = EMV_PIN_MODE_CANCEL;
    func pinRequest(_ mode: EMV_PIN_MODE_Types, key: Data!, pan PAN: Data!, startTO: Int32, intervalTO: Int32, language: String!) {
        _mode = mode;
        let alert = UIAlertView(title: "PIN REQUEST", message:"Please Enter PIN", delegate: self, cancelButtonTitle: "Cancel", otherButtonTitles: "Ok", "")
        alert.alertViewStyle = .secureTextInput
        alert.show()
    }
    
    func alertView(_ alertView: UIAlertView, didDismissWithButtonIndex buttonIndex: Int) {
        if (alertView.title == "Please Select") {
            if buttonIndex == 1 {
                let selectedValue = Int((alertView.textField(at: 0)?.text) ?? "") ?? 0
                alertView.textField(at: 0)?.resignFirstResponder()
                IDT_VP3300.sharedController().emv_callbackResponseLCD(_lcdDisplayMode, selection: UInt8(selectedValue))
                print("Selected Value is \(selectedValue)")
            }
            else {
                IDT_VP3300.sharedController().emv_callbackResponseLCD(0, selection: 0)
            }
        }
        if (alertView.title == "PIN REQUEST") {
            if buttonIndex == 1 {
                let PIN: String? = alertView.textField(at: 0)?.text
                alertView.textField(at: 0)?.resignFirstResponder()
                IDT_VP3300.sharedController().emv_callbackResponsePIN(_mode, ksn: nil, pin: PIN?.data(using: .utf8))
            }
            else {
                IDT_VP3300.sharedController().emv_callbackResponsePIN(EMV_PIN_MODE_CANCEL, ksn: nil, pin: nil)
            }
        }
    }
    @IBAction func searchForBLE(_ sender: UIButton){
        IDT_VP3300.sharedController().device_setBLEFriendlyName(friendlyName.text!)
        IDT_VP3300.sharedController().device_enableBLEDeviceSearch(nil);
        
    }
	
	func bluetoothPickerAlert(_ view: UIAlertView!) {
		view.show()
		let str: String = IDT_Device.sdk_version()
	}
	
    @IBAction func searchForBLEAuto(_ sender: UIButton){
        IDT_VP3300.sharedController().delegate = self
		let svc = CBUUID.init(string: "1820")
		let svcArray = Array<CBUUID>.init(arrayLiteral: svc)
		IDT_VP3300.sharedController()?.scan(forBLEDevices: 2.0, serviceUUIDs: svcArray, options: nil)
        
    }
    
    @IBAction func getFirmwareVersion(_ sender: UIButton) {
		/*
        var result: NSString?
        let rt: RETURN_CODE = IDT_VP3300.sharedController().device_getFirmwareVersion(&result)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Get firmware version: \(result!)", field: resultsTextView)
        } else {
            connection.displayReturnError("Get firmware version", rt: rt, field: resultsTextView)
        }
		
		*/
		let tags = "5657585ADFEF4D5F20"  //two tags 57 (track2), 56 (track1), 58(track3), DFEF4D, 5A (PAN), 5F20 name
			   let tagData = IDTUtility.hex(toData: tags)
			   var retrievedTags: NSDictionary?
			   let rt = IDT_Device.sharedController().retrieveTransactionResult(tagData,  retrievedTags: &retrievedTags)
			   if RETURN_CODE_DO_SUCCESS == rt {
			   //5F20 Name
				   if let tag5F20 = retrievedTags?.value(forKeyPath: "tags.5F20") {
					   NSLog("tag 5F20 is \(tag5F20)")
					   var tag5F20String = String(reflecting: tag5F20)
					 //get rid of < and >
					   tag5F20String = String(tag5F20String.dropFirst())
					   tag5F20String = String(tag5F20String.dropLast())
					   
					
					 }
			}
    }
    
    @IBAction func getL2Version(_ sender: UIButton) {
        var result: NSString?
        let rt: RETURN_CODE = IDT_VP3300.sharedController().emv_getEMVL2Version(&result)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Get EMV L2 version: \(result!)", field: resultsTextView)
        } else {
            connection.displayReturnError("Get EMV L2 version", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func getSerialNumber(_ sender: UIButton) {
        var result: NSString?
        let rt: RETURN_CODE = IDT_VP3300.sharedController().config_getSerialNumber(&result)
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Get serial number: \(result!)", field: resultsTextView)
        } else {
            connection.displayReturnError("Get serial number", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func getICCStatus(_ sender: UIButton) {
        var response: UnsafeMutablePointer<ICCReaderStatus>? = nil
        let rt: RETURN_CODE = IDT_VP3300.sharedController().icc_get(&response)
        
        if RETURN_CODE_DO_SUCCESS != rt {
            connection.displayReturnError("Get ICC status", rt: rt, field: resultsTextView)
            return
        }
        
        var sta: String
        
        if (response?.pointee.iccPower)! {
            sta = "[ICC powered]"
        } else {
            sta = "[ICC power not ready]"
        }
        
        if (response?.pointee.cardSeated)! {
            sta = "\(sta), [Card seated]"
        } else {
            sta = "\(sta), [Card not seated]"
        }
        
        connection.appendMessageToResultsField(sta, field: resultsTextView)
    }
    
    @IBAction func enableCTLS_MSR(_ sender: UIButton) {
        
        //let bytes : [CUnsignedChar] = [0xFF, 0xEE, 0x06, 0x18, 0x9F, 0x22, 0x02, 0x01, 0x00, 0x9F, 0x26, 0x04, 0x00, 0x00, 0x00, 0x02, 0x9F, 0x2B, 0x05, 0x01, 0x00, 0x00, 0x00, 0x00, 0xDF, 0x01, 0x01, 0x01];
        //let test_tags = NSData(bytes: bytes, length: 28)
        
        //let rt : RETURN_CODE = IDT_VP3300.sharedController().device_startTransaction(1.00, amtOther: 0, type: 0, timeout: 30, tags: test_tags as Data!, forceOnline: false, fallback: true)
        
        
        let rt : RETURN_CODE = IDT_VP3300.sharedController().device_startTransaction(1.00, amtOther: 0, type: 0, timeout: 30, tags: nil, forceOnline: false, fallback: true)
        
        
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Enable MSR: OK.", field: resultsTextView)
        } else {
            connection.displayReturnError("Enable CTLS / MSR", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func disableCTLS_MSR(_ sender: UIButton) {
        let rt: RETURN_CODE = IDT_VP3300.sharedController().msr_cancelMSRSwipe()
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Disable MSR: OK.", field: resultsTextView)
        } else {
            connection.displayReturnError("Disable CTLS / MSR", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func testRKI(_ sender: UIButton) {
        let rt = IDT_VP3300.sharedController().device_startRKI()
        
        if RETURN_CODE_DO_SUCCESS == rt {
            connection.appendMessageToResultsField("Test RKI Successful", field: resultsTextView)
        } else {
            connection.displayReturnError("Test RKI", rt: rt, field: resultsTextView)
        }
    }
    
    @IBAction func clearOutputs(_ sender: UIBarButtonItem) {
        connection.clearOutputs(resultsTextView, dataTextView: dataTextView)
    }
}

